# bash completion for astartectl                           -*- shell-script -*-

__astartectl_debug()
{
    if [[ -n ${BASH_COMP_DEBUG_FILE:-} ]]; then
        echo "$*" >> "${BASH_COMP_DEBUG_FILE}"
    fi
}

# Homebrew on Macs have version 1.3 of bash-completion which doesn't include
# _init_completion. This is a very minimal version of that function.
__astartectl_init_completion()
{
    COMPREPLY=()
    _get_comp_words_by_ref "$@" cur prev words cword
}

__astartectl_index_of_word()
{
    local w word=$1
    shift
    index=0
    for w in "$@"; do
        [[ $w = "$word" ]] && return
        index=$((index+1))
    done
    index=-1
}

__astartectl_contains_word()
{
    local w word=$1; shift
    for w in "$@"; do
        [[ $w = "$word" ]] && return
    done
    return 1
}

__astartectl_handle_go_custom_completion()
{
    __astartectl_debug "${FUNCNAME[0]}: cur is ${cur}, words[*] is ${words[*]}, #words[@] is ${#words[@]}"

    local shellCompDirectiveError=1
    local shellCompDirectiveNoSpace=2
    local shellCompDirectiveNoFileComp=4
    local shellCompDirectiveFilterFileExt=8
    local shellCompDirectiveFilterDirs=16

    local out requestComp lastParam lastChar comp directive args

    # Prepare the command to request completions for the program.
    # Calling ${words[0]} instead of directly astartectl allows to handle aliases
    args=("${words[@]:1}")
    requestComp="${words[0]} __completeNoDesc ${args[*]}"

    lastParam=${words[$((${#words[@]}-1))]}
    lastChar=${lastParam:$((${#lastParam}-1)):1}
    __astartectl_debug "${FUNCNAME[0]}: lastParam ${lastParam}, lastChar ${lastChar}"

    if [ -z "${cur}" ] && [ "${lastChar}" != "=" ]; then
        # If the last parameter is complete (there is a space following it)
        # We add an extra empty parameter so we can indicate this to the go method.
        __astartectl_debug "${FUNCNAME[0]}: Adding extra empty parameter"
        requestComp="${requestComp} \"\""
    fi

    __astartectl_debug "${FUNCNAME[0]}: calling ${requestComp}"
    # Use eval to handle any environment variables and such
    out=$(eval "${requestComp}" 2>/dev/null)

    # Extract the directive integer at the very end of the output following a colon (:)
    directive=${out##*:}
    # Remove the directive
    out=${out%:*}
    if [ "${directive}" = "${out}" ]; then
        # There is not directive specified
        directive=0
    fi
    __astartectl_debug "${FUNCNAME[0]}: the completion directive is: ${directive}"
    __astartectl_debug "${FUNCNAME[0]}: the completions are: ${out[*]}"

    if [ $((directive & shellCompDirectiveError)) -ne 0 ]; then
        # Error code.  No completion.
        __astartectl_debug "${FUNCNAME[0]}: received error from custom completion go code"
        return
    else
        if [ $((directive & shellCompDirectiveNoSpace)) -ne 0 ]; then
            if [[ $(type -t compopt) = "builtin" ]]; then
                __astartectl_debug "${FUNCNAME[0]}: activating no space"
                compopt -o nospace
            fi
        fi
        if [ $((directive & shellCompDirectiveNoFileComp)) -ne 0 ]; then
            if [[ $(type -t compopt) = "builtin" ]]; then
                __astartectl_debug "${FUNCNAME[0]}: activating no file completion"
                compopt +o default
            fi
        fi
    fi

    if [ $((directive & shellCompDirectiveFilterFileExt)) -ne 0 ]; then
        # File extension filtering
        local fullFilter filter filteringCmd
        # Do not use quotes around the $out variable or else newline
        # characters will be kept.
        for filter in ${out[*]}; do
            fullFilter+="$filter|"
        done

        filteringCmd="_filedir $fullFilter"
        __astartectl_debug "File filtering command: $filteringCmd"
        $filteringCmd
    elif [ $((directive & shellCompDirectiveFilterDirs)) -ne 0 ]; then
        # File completion for directories only
        local subdir
        # Use printf to strip any trailing newline
        subdir=$(printf "%s" "${out[0]}")
        if [ -n "$subdir" ]; then
            __astartectl_debug "Listing directories in $subdir"
            __astartectl_handle_subdirs_in_dir_flag "$subdir"
        else
            __astartectl_debug "Listing directories in ."
            _filedir -d
        fi
    else
        while IFS='' read -r comp; do
            COMPREPLY+=("$comp")
        done < <(compgen -W "${out[*]}" -- "$cur")
    fi
}

__astartectl_handle_reply()
{
    __astartectl_debug "${FUNCNAME[0]}"
    local comp
    case $cur in
        -*)
            if [[ $(type -t compopt) = "builtin" ]]; then
                compopt -o nospace
            fi
            local allflags
            if [ ${#must_have_one_flag[@]} -ne 0 ]; then
                allflags=("${must_have_one_flag[@]}")
            else
                allflags=("${flags[*]} ${two_word_flags[*]}")
            fi
            while IFS='' read -r comp; do
                COMPREPLY+=("$comp")
            done < <(compgen -W "${allflags[*]}" -- "$cur")
            if [[ $(type -t compopt) = "builtin" ]]; then
                [[ "${COMPREPLY[0]}" == *= ]] || compopt +o nospace
            fi

            # complete after --flag=abc
            if [[ $cur == *=* ]]; then
                if [[ $(type -t compopt) = "builtin" ]]; then
                    compopt +o nospace
                fi

                local index flag
                flag="${cur%=*}"
                __astartectl_index_of_word "${flag}" "${flags_with_completion[@]}"
                COMPREPLY=()
                if [[ ${index} -ge 0 ]]; then
                    PREFIX=""
                    cur="${cur#*=}"
                    ${flags_completion[${index}]}
                    if [ -n "${ZSH_VERSION:-}" ]; then
                        # zsh completion needs --flag= prefix
                        eval "COMPREPLY=( \"\${COMPREPLY[@]/#/${flag}=}\" )"
                    fi
                fi
            fi

            if [[ -z "${flag_parsing_disabled}" ]]; then
                # If flag parsing is enabled, we have completed the flags and can return.
                # If flag parsing is disabled, we may not know all (or any) of the flags, so we fallthrough
                # to possibly call handle_go_custom_completion.
                return 0;
            fi
            ;;
    esac

    # check if we are handling a flag with special work handling
    local index
    __astartectl_index_of_word "${prev}" "${flags_with_completion[@]}"
    if [[ ${index} -ge 0 ]]; then
        ${flags_completion[${index}]}
        return
    fi

    # we are parsing a flag and don't have a special handler, no completion
    if [[ ${cur} != "${words[cword]}" ]]; then
        return
    fi

    local completions
    completions=("${commands[@]}")
    if [[ ${#must_have_one_noun[@]} -ne 0 ]]; then
        completions+=("${must_have_one_noun[@]}")
    elif [[ -n "${has_completion_function}" ]]; then
        # if a go completion function is provided, defer to that function
        __astartectl_handle_go_custom_completion
    fi
    if [[ ${#must_have_one_flag[@]} -ne 0 ]]; then
        completions+=("${must_have_one_flag[@]}")
    fi
    while IFS='' read -r comp; do
        COMPREPLY+=("$comp")
    done < <(compgen -W "${completions[*]}" -- "$cur")

    if [[ ${#COMPREPLY[@]} -eq 0 && ${#noun_aliases[@]} -gt 0 && ${#must_have_one_noun[@]} -ne 0 ]]; then
        while IFS='' read -r comp; do
            COMPREPLY+=("$comp")
        done < <(compgen -W "${noun_aliases[*]}" -- "$cur")
    fi

    if [[ ${#COMPREPLY[@]} -eq 0 ]]; then
        if declare -F __astartectl_custom_func >/dev/null; then
            # try command name qualified custom func
            __astartectl_custom_func
        else
            # otherwise fall back to unqualified for compatibility
            declare -F __custom_func >/dev/null && __custom_func
        fi
    fi

    # available in bash-completion >= 2, not always present on macOS
    if declare -F __ltrim_colon_completions >/dev/null; then
        __ltrim_colon_completions "$cur"
    fi

    # If there is only 1 completion and it is a flag with an = it will be completed
    # but we don't want a space after the =
    if [[ "${#COMPREPLY[@]}" -eq "1" ]] && [[ $(type -t compopt) = "builtin" ]] && [[ "${COMPREPLY[0]}" == --*= ]]; then
       compopt -o nospace
    fi
}

# The arguments should be in the form "ext1|ext2|extn"
__astartectl_handle_filename_extension_flag()
{
    local ext="$1"
    _filedir "@(${ext})"
}

__astartectl_handle_subdirs_in_dir_flag()
{
    local dir="$1"
    pushd "${dir}" >/dev/null 2>&1 && _filedir -d && popd >/dev/null 2>&1 || return
}

__astartectl_handle_flag()
{
    __astartectl_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    # if a command required a flag, and we found it, unset must_have_one_flag()
    local flagname=${words[c]}
    local flagvalue=""
    # if the word contained an =
    if [[ ${words[c]} == *"="* ]]; then
        flagvalue=${flagname#*=} # take in as flagvalue after the =
        flagname=${flagname%=*} # strip everything after the =
        flagname="${flagname}=" # but put the = back
    fi
    __astartectl_debug "${FUNCNAME[0]}: looking for ${flagname}"
    if __astartectl_contains_word "${flagname}" "${must_have_one_flag[@]}"; then
        must_have_one_flag=()
    fi

    # if you set a flag which only applies to this command, don't show subcommands
    if __astartectl_contains_word "${flagname}" "${local_nonpersistent_flags[@]}"; then
      commands=()
    fi

    # keep flag value with flagname as flaghash
    # flaghash variable is an associative array which is only supported in bash > 3.
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        if [ -n "${flagvalue}" ] ; then
            flaghash[${flagname}]=${flagvalue}
        elif [ -n "${words[ $((c+1)) ]}" ] ; then
            flaghash[${flagname}]=${words[ $((c+1)) ]}
        else
            flaghash[${flagname}]="true" # pad "true" for bool flag
        fi
    fi

    # skip the argument to a two word flag
    if [[ ${words[c]} != *"="* ]] && __astartectl_contains_word "${words[c]}" "${two_word_flags[@]}"; then
        __astartectl_debug "${FUNCNAME[0]}: found a flag ${words[c]}, skip the next argument"
        c=$((c+1))
        # if we are looking for a flags value, don't show commands
        if [[ $c -eq $cword ]]; then
            commands=()
        fi
    fi

    c=$((c+1))

}

__astartectl_handle_noun()
{
    __astartectl_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    if __astartectl_contains_word "${words[c]}" "${must_have_one_noun[@]}"; then
        must_have_one_noun=()
    elif __astartectl_contains_word "${words[c]}" "${noun_aliases[@]}"; then
        must_have_one_noun=()
    fi

    nouns+=("${words[c]}")
    c=$((c+1))
}

__astartectl_handle_command()
{
    __astartectl_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    local next_command
    if [[ -n ${last_command} ]]; then
        next_command="_${last_command}_${words[c]//:/__}"
    else
        if [[ $c -eq 0 ]]; then
            next_command="_astartectl_root_command"
        else
            next_command="_${words[c]//:/__}"
        fi
    fi
    c=$((c+1))
    __astartectl_debug "${FUNCNAME[0]}: looking for ${next_command}"
    declare -F "$next_command" >/dev/null && $next_command
}

__astartectl_handle_word()
{
    if [[ $c -ge $cword ]]; then
        __astartectl_handle_reply
        return
    fi
    __astartectl_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
    if [[ "${words[c]}" == -* ]]; then
        __astartectl_handle_flag
    elif __astartectl_contains_word "${words[c]}" "${commands[@]}"; then
        __astartectl_handle_command
    elif [[ $c -eq 0 ]]; then
        __astartectl_handle_command
    elif __astartectl_contains_word "${words[c]}" "${command_aliases[@]}"; then
        # aliashash variable is an associative array which is only supported in bash > 3.
        if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
            words[c]=${aliashash[${words[c]}]}
            __astartectl_handle_command
        else
            __astartectl_handle_noun
        fi
    else
        __astartectl_handle_noun
    fi
    __astartectl_handle_word
}

_astartectl_appengine_devices_aliases_add()
{
    last_command="astartectl_appengine_devices_aliases_add"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_aliases_list()
{
    last_command="astartectl_appengine_devices_aliases_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_aliases_remove()
{
    last_command="astartectl_appengine_devices_aliases_remove"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_aliases()
{
    last_command="astartectl_appengine_devices_aliases"

    command_aliases=()

    commands=()
    commands+=("add")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("remove")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("rm")
        aliashash["rm"]="remove"
    fi

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_attributes_list()
{
    last_command="astartectl_appengine_devices_attributes_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_attributes_remove()
{
    last_command="astartectl_appengine_devices_attributes_remove"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_attributes_set()
{
    last_command="astartectl_appengine_devices_attributes_set"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_attributes()
{
    last_command="astartectl_appengine_devices_attributes"

    command_aliases=()

    commands=()
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("remove")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("rm")
        aliashash["rm"]="remove"
    fi
    commands+=("set")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_credentials_inhibit()
{
    last_command="astartectl_appengine_devices_credentials_inhibit"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_credentials()
{
    last_command="astartectl_appengine_devices_credentials"

    command_aliases=()

    commands=()
    commands+=("inhibit")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_data-snapshot()
{
    last_command="astartectl_appengine_devices_data-snapshot"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--interface-type=")
    two_word_flags+=("--interface-type")
    local_nonpersistent_flags+=("--interface-type")
    local_nonpersistent_flags+=("--interface-type=")
    flags+=("--output=")
    two_word_flags+=("--output")
    two_word_flags+=("-o")
    local_nonpersistent_flags+=("--output")
    local_nonpersistent_flags+=("--output=")
    local_nonpersistent_flags+=("-o")
    flags+=("--skip-realm-management-checks")
    local_nonpersistent_flags+=("--skip-realm-management-checks")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_get-samples()
{
    last_command="astartectl_appengine_devices_get-samples"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--aggregate")
    local_nonpersistent_flags+=("--aggregate")
    flags+=("--ascending")
    local_nonpersistent_flags+=("--ascending")
    flags+=("--count=")
    two_word_flags+=("--count")
    two_word_flags+=("-c")
    local_nonpersistent_flags+=("--count")
    local_nonpersistent_flags+=("--count=")
    local_nonpersistent_flags+=("-c")
    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--output=")
    two_word_flags+=("--output")
    two_word_flags+=("-o")
    local_nonpersistent_flags+=("--output")
    local_nonpersistent_flags+=("--output=")
    local_nonpersistent_flags+=("-o")
    flags+=("--since=")
    two_word_flags+=("--since")
    local_nonpersistent_flags+=("--since")
    local_nonpersistent_flags+=("--since=")
    flags+=("--skip-realm-management-checks")
    local_nonpersistent_flags+=("--skip-realm-management-checks")
    flags+=("--to=")
    two_word_flags+=("--to")
    local_nonpersistent_flags+=("--to")
    local_nonpersistent_flags+=("--to=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_list()
{
    last_command="astartectl_appengine_devices_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--details")
    flags+=("-d")
    local_nonpersistent_flags+=("--details")
    local_nonpersistent_flags+=("-d")
    flags+=("--filter=")
    two_word_flags+=("--filter")
    two_word_flags+=("-f")
    local_nonpersistent_flags+=("--filter")
    local_nonpersistent_flags+=("--filter=")
    local_nonpersistent_flags+=("-f")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_publish-datastream()
{
    last_command="astartectl_appengine_devices_publish-datastream"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--interface-type=")
    two_word_flags+=("--interface-type")
    local_nonpersistent_flags+=("--interface-type")
    local_nonpersistent_flags+=("--interface-type=")
    flags+=("--payload-type=")
    two_word_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type=")
    flags+=("--skip-realm-management-checks")
    local_nonpersistent_flags+=("--skip-realm-management-checks")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_send-data()
{
    last_command="astartectl_appengine_devices_send-data"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--interface-type=")
    two_word_flags+=("--interface-type")
    local_nonpersistent_flags+=("--interface-type")
    local_nonpersistent_flags+=("--interface-type=")
    flags+=("--payload-type=")
    two_word_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type=")
    flags+=("--skip-realm-management-checks")
    local_nonpersistent_flags+=("--skip-realm-management-checks")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_set-property()
{
    last_command="astartectl_appengine_devices_set-property"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--payload-type=")
    two_word_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type=")
    flags+=("--skip-realm-management-checks")
    local_nonpersistent_flags+=("--skip-realm-management-checks")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_show()
{
    last_command="astartectl_appengine_devices_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices_unset-property()
{
    last_command="astartectl_appengine_devices_unset-property"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--payload-type=")
    two_word_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type")
    local_nonpersistent_flags+=("--payload-type=")
    flags+=("--skip-realm-management-checks")
    local_nonpersistent_flags+=("--skip-realm-management-checks")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_devices()
{
    last_command="astartectl_appengine_devices"

    command_aliases=()

    commands=()
    commands+=("aliases")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("alias")
        aliashash["alias"]="aliases"
    fi
    commands+=("attributes")
    commands+=("credentials")
    commands+=("data-snapshot")
    commands+=("get-samples")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("publish-datastream")
    commands+=("send-data")
    commands+=("set-property")
    commands+=("show")
    commands+=("unset-property")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups_create()
{
    last_command="astartectl_appengine_groups_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups_devices_add()
{
    last_command="astartectl_appengine_groups_devices_add"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups_devices_list()
{
    last_command="astartectl_appengine_groups_devices_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups_devices_remove()
{
    last_command="astartectl_appengine_groups_devices_remove"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force-id-type=")
    two_word_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type")
    local_nonpersistent_flags+=("--force-id-type=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups_devices()
{
    last_command="astartectl_appengine_groups_devices"

    command_aliases=()

    commands=()
    commands+=("add")
    commands+=("list")
    commands+=("remove")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups_list()
{
    last_command="astartectl_appengine_groups_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_groups()
{
    last_command="astartectl_appengine_groups"

    command_aliases=()

    commands=()
    commands+=("create")
    commands+=("devices")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("device")
        aliashash["device"]="devices"
    fi
    commands+=("list")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_stats_devices()
{
    last_command="astartectl_appengine_stats_devices"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine_stats()
{
    last_command="astartectl_appengine_stats"

    command_aliases=()

    commands=()
    commands+=("devices")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("device")
        aliashash["device"]="devices"
    fi

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_appengine()
{
    last_command="astartectl_appengine"

    command_aliases=()

    commands=()
    commands+=("devices")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("device")
        aliashash["device"]="devices"
    fi
    commands+=("groups")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("group")
        aliashash["group"]="groups"
    fi
    commands+=("stats")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("stat")
        aliashash["stat"]="stats"
        command_aliases+=("statistics")
        aliashash["statistics"]="stats"
    fi

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_install-operator()
{
    last_command="astartectl_cluster_install-operator"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_instances_fetch-housekeeping-key()
{
    last_command="astartectl_cluster_instances_fetch-housekeeping-key"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--output=")
    two_word_flags+=("--output")
    two_word_flags+=("-o")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--namespace=")
    two_word_flags+=("--namespace")
    two_word_flags+=("-n")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_instances_get-cluster-config()
{
    last_command="astartectl_cluster_instances_get-cluster-config"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--namespace=")
    two_word_flags+=("--namespace")
    two_word_flags+=("-n")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_instances_migrate_replace-voyager()
{
    last_command="astartectl_cluster_instances_migrate_replace-voyager"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--ingress-name=")
    two_word_flags+=("--ingress-name")
    flags+=("--operator-name=")
    two_word_flags+=("--operator-name")
    flags+=("--operator-namespace=")
    two_word_flags+=("--operator-namespace")
    flags+=("--out=")
    two_word_flags+=("--out")
    two_word_flags+=("-o")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--namespace=")
    two_word_flags+=("--namespace")
    two_word_flags+=("-n")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_instances_migrate_storage-version()
{
    last_command="astartectl_cluster_instances_migrate_storage-version"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--namespace=")
    two_word_flags+=("--namespace")
    two_word_flags+=("-n")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_instances_migrate()
{
    last_command="astartectl_cluster_instances_migrate"

    command_aliases=()

    commands=()
    commands+=("replace-voyager")
    commands+=("storage-version")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--namespace=")
    two_word_flags+=("--namespace")
    two_word_flags+=("-n")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_instances()
{
    last_command="astartectl_cluster_instances"

    command_aliases=()

    commands=()
    commands+=("fetch-housekeeping-key")
    commands+=("get-cluster-config")
    commands+=("migrate")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--namespace=")
    two_word_flags+=("--namespace")
    two_word_flags+=("-n")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_show()
{
    last_command="astartectl_cluster_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--operator-name=")
    two_word_flags+=("--operator-name")
    flags+=("--operator-namespace=")
    two_word_flags+=("--operator-namespace")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_uninstall-operator()
{
    last_command="astartectl_cluster_uninstall-operator"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster_upgrade-operator()
{
    last_command="astartectl_cluster_upgrade-operator"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_cluster()
{
    last_command="astartectl_cluster"

    command_aliases=()

    commands=()
    commands+=("install-operator")
    commands+=("instances")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("instance")
        aliashash["instance"]="instances"
    fi
    commands+=("show")
    commands+=("uninstall-operator")
    commands+=("upgrade-operator")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--kubeconfig=")
    two_word_flags+=("--kubeconfig")
    two_word_flags+=("-k")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_completion_bash()
{
    last_command="astartectl_completion_bash"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--help")
    flags+=("-h")
    local_nonpersistent_flags+=("--help")
    local_nonpersistent_flags+=("-h")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_completion_fish()
{
    last_command="astartectl_completion_fish"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_completion_zsh()
{
    last_command="astartectl_completion_zsh"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_completion()
{
    last_command="astartectl_completion"

    command_aliases=()

    commands=()
    commands+=("bash")
    commands+=("fish")
    commands+=("zsh")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters_create()
{
    last_command="astartectl_config_clusters_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--api-url=")
    two_word_flags+=("--api-url")
    local_nonpersistent_flags+=("--api-url")
    local_nonpersistent_flags+=("--api-url=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    local_nonpersistent_flags+=("--appengine-url")
    local_nonpersistent_flags+=("--appengine-url=")
    flags+=("--flow-url=")
    two_word_flags+=("--flow-url")
    local_nonpersistent_flags+=("--flow-url")
    local_nonpersistent_flags+=("--flow-url=")
    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--housekeeping-key")
    local_nonpersistent_flags+=("--housekeeping-key=")
    local_nonpersistent_flags+=("-k")
    flags+=("--housekeeping-token=")
    two_word_flags+=("--housekeeping-token")
    local_nonpersistent_flags+=("--housekeeping-token")
    local_nonpersistent_flags+=("--housekeeping-token=")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    local_nonpersistent_flags+=("--housekeeping-url")
    local_nonpersistent_flags+=("--housekeeping-url=")
    flags+=("--pairing-url=")
    two_word_flags+=("--pairing-url")
    local_nonpersistent_flags+=("--pairing-url")
    local_nonpersistent_flags+=("--pairing-url=")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    local_nonpersistent_flags+=("--realm-management-url")
    local_nonpersistent_flags+=("--realm-management-url=")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters_delete()
{
    last_command="astartectl_config_clusters_delete"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters_get-housekeeping-key()
{
    last_command="astartectl_config_clusters_get-housekeeping-key"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--output=")
    two_word_flags+=("--output")
    two_word_flags+=("-o")
    local_nonpersistent_flags+=("--output")
    local_nonpersistent_flags+=("--output=")
    local_nonpersistent_flags+=("-o")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters_list()
{
    last_command="astartectl_config_clusters_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters_show()
{
    last_command="astartectl_config_clusters_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters_update()
{
    last_command="astartectl_config_clusters_update"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--api-url=")
    two_word_flags+=("--api-url")
    local_nonpersistent_flags+=("--api-url")
    local_nonpersistent_flags+=("--api-url=")
    flags+=("--appengine-url=")
    two_word_flags+=("--appengine-url")
    local_nonpersistent_flags+=("--appengine-url")
    local_nonpersistent_flags+=("--appengine-url=")
    flags+=("--flow-url=")
    two_word_flags+=("--flow-url")
    local_nonpersistent_flags+=("--flow-url")
    local_nonpersistent_flags+=("--flow-url=")
    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--housekeeping-key")
    local_nonpersistent_flags+=("--housekeeping-key=")
    local_nonpersistent_flags+=("-k")
    flags+=("--housekeeping-token=")
    two_word_flags+=("--housekeeping-token")
    local_nonpersistent_flags+=("--housekeeping-token")
    local_nonpersistent_flags+=("--housekeeping-token=")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    local_nonpersistent_flags+=("--housekeeping-url")
    local_nonpersistent_flags+=("--housekeeping-url=")
    flags+=("--pairing-url=")
    two_word_flags+=("--pairing-url")
    local_nonpersistent_flags+=("--pairing-url")
    local_nonpersistent_flags+=("--pairing-url=")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    local_nonpersistent_flags+=("--realm-management-url")
    local_nonpersistent_flags+=("--realm-management-url=")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_clusters()
{
    last_command="astartectl_config_clusters"

    command_aliases=()

    commands=()
    commands+=("create")
    commands+=("delete")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("del")
        aliashash["del"]="delete"
    fi
    commands+=("get-housekeeping-key")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("show")
    commands+=("update")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts_create()
{
    last_command="astartectl_config_contexts_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--activate")
    flags+=("-a")
    local_nonpersistent_flags+=("--activate")
    local_nonpersistent_flags+=("-a")
    flags+=("--cluster=")
    two_word_flags+=("--cluster")
    two_word_flags+=("-c")
    local_nonpersistent_flags+=("--cluster")
    local_nonpersistent_flags+=("--cluster=")
    local_nonpersistent_flags+=("-c")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    local_nonpersistent_flags+=("--realm-name")
    local_nonpersistent_flags+=("--realm-name=")
    local_nonpersistent_flags+=("-r")
    flags+=("--realm-private-key=")
    two_word_flags+=("--realm-private-key")
    flags_with_completion+=("--realm-private-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--realm-private-key")
    local_nonpersistent_flags+=("--realm-private-key=")
    local_nonpersistent_flags+=("-k")
    flags+=("--realm-token=")
    two_word_flags+=("--realm-token")
    local_nonpersistent_flags+=("--realm-token")
    local_nonpersistent_flags+=("--realm-token=")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_flag+=("--cluster=")
    must_have_one_flag+=("-c")
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts_delete()
{
    last_command="astartectl_config_contexts_delete"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts_get-realm-key()
{
    last_command="astartectl_config_contexts_get-realm-key"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--output=")
    two_word_flags+=("--output")
    two_word_flags+=("-o")
    local_nonpersistent_flags+=("--output")
    local_nonpersistent_flags+=("--output=")
    local_nonpersistent_flags+=("-o")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts_list()
{
    last_command="astartectl_config_contexts_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts_show()
{
    last_command="astartectl_config_contexts_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts_update()
{
    last_command="astartectl_config_contexts_update"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--activate")
    flags+=("-a")
    local_nonpersistent_flags+=("--activate")
    local_nonpersistent_flags+=("-a")
    flags+=("--cluster=")
    two_word_flags+=("--cluster")
    two_word_flags+=("-c")
    local_nonpersistent_flags+=("--cluster")
    local_nonpersistent_flags+=("--cluster=")
    local_nonpersistent_flags+=("-c")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    local_nonpersistent_flags+=("--realm-name")
    local_nonpersistent_flags+=("--realm-name=")
    local_nonpersistent_flags+=("-r")
    flags+=("--realm-private-key=")
    two_word_flags+=("--realm-private-key")
    flags_with_completion+=("--realm-private-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--realm-private-key")
    local_nonpersistent_flags+=("--realm-private-key=")
    local_nonpersistent_flags+=("-k")
    flags+=("--realm-token=")
    two_word_flags+=("--realm-token")
    local_nonpersistent_flags+=("--realm-token")
    local_nonpersistent_flags+=("--realm-token=")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_contexts()
{
    last_command="astartectl_config_contexts"

    command_aliases=()

    commands=()
    commands+=("create")
    commands+=("delete")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("del")
        aliashash["del"]="delete"
    fi
    commands+=("get-realm-key")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("show")
    commands+=("update")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_current-cluster()
{
    last_command="astartectl_config_current-cluster"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_current-context()
{
    last_command="astartectl_config_current-context"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_export()
{
    last_command="astartectl_config_export"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--clusters=")
    two_word_flags+=("--clusters")
    local_nonpersistent_flags+=("--clusters")
    local_nonpersistent_flags+=("--clusters=")
    flags+=("--contexts=")
    two_word_flags+=("--contexts")
    local_nonpersistent_flags+=("--contexts")
    local_nonpersistent_flags+=("--contexts=")
    flags+=("--output=")
    two_word_flags+=("--output")
    two_word_flags+=("-o")
    local_nonpersistent_flags+=("--output")
    local_nonpersistent_flags+=("--output=")
    local_nonpersistent_flags+=("-o")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_import()
{
    last_command="astartectl_config_import"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--clusters=")
    two_word_flags+=("--clusters")
    local_nonpersistent_flags+=("--clusters")
    local_nonpersistent_flags+=("--clusters=")
    flags+=("--contexts=")
    two_word_flags+=("--contexts")
    local_nonpersistent_flags+=("--contexts")
    local_nonpersistent_flags+=("--contexts=")
    flags+=("--overwrite")
    local_nonpersistent_flags+=("--overwrite")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config_set-current-context()
{
    last_command="astartectl_config_set-current-context"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_config()
{
    last_command="astartectl_config"

    command_aliases=()

    commands=()
    commands+=("clusters")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("cluster")
        aliashash["cluster"]="clusters"
    fi
    commands+=("contexts")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("context")
        aliashash["context"]="contexts"
    fi
    commands+=("current-cluster")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("get-current-cluster")
        aliashash["get-current-cluster"]="current-cluster"
    fi
    commands+=("current-context")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("get-current-context")
        aliashash["get-current-context"]="current-context"
    fi
    commands+=("export")
    commands+=("import")
    commands+=("set-current-context")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("use-context")
        aliashash["use-context"]="set-current-context"
    fi

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_help()
{
    last_command="astartectl_help"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    has_completion_function=1
    noun_aliases=()
}

_astartectl_housekeeping_realms_create()
{
    last_command="astartectl_housekeeping_realms_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--datacenter-replication=")
    two_word_flags+=("--datacenter-replication")
    two_word_flags+=("-d")
    local_nonpersistent_flags+=("--datacenter-replication")
    local_nonpersistent_flags+=("--datacenter-replication=")
    local_nonpersistent_flags+=("-d")
    flags+=("--non-interactive")
    flags+=("-y")
    flags+=("--realm-private-key=")
    two_word_flags+=("--realm-private-key")
    flags_with_completion+=("--realm-private-key")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--realm-private-key")
    local_nonpersistent_flags+=("--realm-private-key=")
    flags+=("--realm-public-key=")
    two_word_flags+=("--realm-public-key")
    flags_with_completion+=("--realm-public-key")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--realm-public-key")
    local_nonpersistent_flags+=("--realm-public-key=")
    flags+=("--replication-factor=")
    two_word_flags+=("--replication-factor")
    two_word_flags+=("-r")
    local_nonpersistent_flags+=("--replication-factor")
    local_nonpersistent_flags+=("--replication-factor=")
    local_nonpersistent_flags+=("-r")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    flags+=("--ignore-ssl-errors")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_housekeeping_realms_list()
{
    last_command="astartectl_housekeeping_realms_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    flags+=("--ignore-ssl-errors")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_housekeeping_realms_show()
{
    last_command="astartectl_housekeeping_realms_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    flags+=("--ignore-ssl-errors")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_housekeeping_realms()
{
    last_command="astartectl_housekeeping_realms"

    command_aliases=()

    commands=()
    commands+=("create")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("show")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    flags+=("--ignore-ssl-errors")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_housekeeping()
{
    last_command="astartectl_housekeeping"

    command_aliases=()

    commands=()
    commands+=("realms")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("realm")
        aliashash["realm"]="realms"
    fi

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--housekeeping-key=")
    two_word_flags+=("--housekeeping-key")
    flags_with_completion+=("--housekeeping-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--housekeeping-url=")
    two_word_flags+=("--housekeeping-url")
    flags+=("--to-curl")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_pairing_agent_register()
{
    last_command="astartectl_pairing_agent_register"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--compact-output")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--pairing-url=")
    two_word_flags+=("--pairing-url")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_pairing_agent_unregister()
{
    last_command="astartectl_pairing_agent_unregister"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--non-interactive")
    flags+=("-y")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--pairing-url=")
    two_word_flags+=("--pairing-url")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_pairing_agent()
{
    last_command="astartectl_pairing_agent"

    command_aliases=()

    commands=()
    commands+=("register")
    commands+=("unregister")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--pairing-url=")
    two_word_flags+=("--pairing-url")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_pairing()
{
    last_command="astartectl_pairing"

    command_aliases=()

    commands=()
    commands+=("agent")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--pairing-url=")
    two_word_flags+=("--pairing-url")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_delete()
{
    last_command="astartectl_realm-management_interfaces_delete"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_install()
{
    last_command="astartectl_realm-management_interfaces_install"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_list()
{
    last_command="astartectl_realm-management_interfaces_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_save()
{
    last_command="astartectl_realm-management_interfaces_save"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_show()
{
    last_command="astartectl_realm-management_interfaces_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_sync()
{
    last_command="astartectl_realm-management_interfaces_sync"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--non-interactive")
    flags+=("-y")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_update()
{
    last_command="astartectl_realm-management_interfaces_update"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces_versions()
{
    last_command="astartectl_realm-management_interfaces_versions"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_interfaces()
{
    last_command="astartectl_realm-management_interfaces"

    command_aliases=()

    commands=()
    commands+=("delete")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("del")
        aliashash["del"]="delete"
    fi
    commands+=("install")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("save")
    commands+=("show")
    commands+=("sync")
    commands+=("update")
    commands+=("versions")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers_delete()
{
    last_command="astartectl_realm-management_triggers_delete"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers_install()
{
    last_command="astartectl_realm-management_triggers_install"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers_list()
{
    last_command="astartectl_realm-management_triggers_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers_save()
{
    last_command="astartectl_realm-management_triggers_save"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers_show()
{
    last_command="astartectl_realm-management_triggers_show"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers_sync()
{
    last_command="astartectl_realm-management_triggers_sync"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    local_nonpersistent_flags+=("--force")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management_triggers()
{
    last_command="astartectl_realm-management_triggers"

    command_aliases=()

    commands=()
    commands+=("delete")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("del")
        aliashash["del"]="delete"
    fi
    commands+=("install")
    commands+=("list")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("ls")
        aliashash["ls"]="list"
    fi
    commands+=("save")
    commands+=("show")
    commands+=("sync")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_realm-management()
{
    last_command="astartectl_realm-management"

    command_aliases=()

    commands=()
    commands+=("interfaces")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("interface")
        aliashash["interface"]="interfaces"
    fi
    commands+=("triggers")
    if [[ -z "${BASH_VERSION:-}" || "${BASH_VERSINFO[0]:-}" -gt 3 ]]; then
        command_aliases+=("trigger")
        aliashash["trigger"]="triggers"
    fi

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--realm-key=")
    two_word_flags+=("--realm-key")
    flags_with_completion+=("--realm-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    flags+=("--realm-management-url=")
    two_word_flags+=("--realm-management-url")
    flags+=("--realm-name=")
    two_word_flags+=("--realm-name")
    two_word_flags+=("-r")
    flags+=("--to-curl")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id_compute-from-bytes()
{
    last_command="astartectl_utils_device-id_compute-from-bytes"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id_compute-from-string()
{
    last_command="astartectl_utils_device-id_compute-from-string"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id_from-uuid()
{
    last_command="astartectl_utils_device-id_from-uuid"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id_generate-random()
{
    last_command="astartectl_utils_device-id_generate-random"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id_to-uuid()
{
    last_command="astartectl_utils_device-id_to-uuid"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id_validate()
{
    last_command="astartectl_utils_device-id_validate"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_device-id()
{
    last_command="astartectl_utils_device-id"

    command_aliases=()

    commands=()
    commands+=("compute-from-bytes")
    commands+=("compute-from-string")
    commands+=("from-uuid")
    commands+=("generate-random")
    commands+=("to-uuid")
    commands+=("validate")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_gen-jwt()
{
    last_command="astartectl_utils_gen-jwt"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--claims=")
    two_word_flags+=("--claims")
    two_word_flags+=("-c")
    local_nonpersistent_flags+=("--claims")
    local_nonpersistent_flags+=("--claims=")
    local_nonpersistent_flags+=("-c")
    flags+=("--expiry=")
    two_word_flags+=("--expiry")
    two_word_flags+=("-e")
    local_nonpersistent_flags+=("--expiry")
    local_nonpersistent_flags+=("--expiry=")
    local_nonpersistent_flags+=("-e")
    flags+=("--private-key=")
    two_word_flags+=("--private-key")
    flags_with_completion+=("--private-key")
    flags_completion+=("_filedir")
    two_word_flags+=("-k")
    flags_with_completion+=("-k")
    flags_completion+=("_filedir")
    local_nonpersistent_flags+=("--private-key")
    local_nonpersistent_flags+=("--private-key=")
    local_nonpersistent_flags+=("-k")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    must_have_one_noun+=("appengine")
    must_have_one_noun+=("channels")
    must_have_one_noun+=("housekeeping")
    must_have_one_noun+=("pairing")
    must_have_one_noun+=("realm-management")
    noun_aliases=()
}

_astartectl_utils_gen-keypair()
{
    last_command="astartectl_utils_gen-keypair"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_interfaces_validate()
{
    last_command="astartectl_utils_interfaces_validate"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_interfaces()
{
    last_command="astartectl_utils_interfaces"

    command_aliases=()

    commands=()
    commands+=("validate")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_show-jwt-claims()
{
    last_command="astartectl_utils_show-jwt-claims"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--pretty")
    flags+=("-p")
    local_nonpersistent_flags+=("--pretty")
    local_nonpersistent_flags+=("-p")
    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_triggers_validate()
{
    last_command="astartectl_utils_triggers_validate"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils_triggers()
{
    last_command="astartectl_utils_triggers"

    command_aliases=()

    commands=()
    commands+=("validate")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_utils()
{
    last_command="astartectl_utils"

    command_aliases=()

    commands=()
    commands+=("device-id")
    commands+=("gen-jwt")
    commands+=("gen-keypair")
    commands+=("interfaces")
    commands+=("show-jwt-claims")
    commands+=("triggers")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_version()
{
    last_command="astartectl_version"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_astartectl_root_command()
{
    last_command="astartectl"

    command_aliases=()

    commands=()
    commands+=("appengine")
    commands+=("cluster")
    commands+=("completion")
    commands+=("config")
    commands+=("help")
    commands+=("housekeeping")
    commands+=("pairing")
    commands+=("realm-management")
    commands+=("utils")
    commands+=("version")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--astarte-url=")
    two_word_flags+=("--astarte-url")
    two_word_flags+=("-u")
    flags+=("--config-dir=")
    two_word_flags+=("--config-dir")
    flags+=("--context=")
    two_word_flags+=("--context")
    flags+=("--ignore-ssl-errors")
    flags+=("--token=")
    two_word_flags+=("--token")
    two_word_flags+=("-t")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

__start_astartectl()
{
    local cur prev words cword split
    declare -A flaghash 2>/dev/null || :
    declare -A aliashash 2>/dev/null || :
    if declare -F _init_completion >/dev/null 2>&1; then
        _init_completion -s || return
    else
        __astartectl_init_completion -n "=" || return
    fi

    local c=0
    local flag_parsing_disabled=
    local flags=()
    local two_word_flags=()
    local local_nonpersistent_flags=()
    local flags_with_completion=()
    local flags_completion=()
    local commands=("astartectl")
    local command_aliases=()
    local must_have_one_flag=()
    local must_have_one_noun=()
    local has_completion_function=""
    local last_command=""
    local nouns=()
    local noun_aliases=()

    __astartectl_handle_word
}

if [[ $(type -t compopt) = "builtin" ]]; then
    complete -o default -F __start_astartectl astartectl
else
    complete -o default -o nospace -F __start_astartectl astartectl
fi

# ex: ts=4 sw=4 et filetype=sh
